@@ -1,7 +1,9 @@
+pub mod cli;
 pub mod monitor;
 mod statusbar;
 mod statusblock;
 pub mod threadpool;
+pub mod utils;
 
 pub use statusbar::{StatusBar, StatusBarBuilder};
 pub use statusblock::{StatusBlock, StatusBlockBuilder};
