@@ -1,27 +1,32 @@
+use std::sync::{mpsc, Arc, Mutex};
 use std::thread::{self, JoinHandle};
 
 use crate::monitor::{Command, MonitorSender};
 
-pub type ResultsSender<T> = flume::Sender<ResultPacket<T>>;
-pub type ResultsReceiver<T> = flume::Receiver<ResultPacket<T>>;
-pub type JobsSender<T> = flume::Sender<Message<T>>;
-pub type JobsReceiver<T> = flume::Receiver<Message<T>>;
+pub type ResultsSender<T> = mpsc::SyncSender<ResultPacket<T>>;
+pub type ResultsReceiver<T> = mpsc::Receiver<ResultPacket<T>>;
+pub type JobsSender<T> = mpsc::SyncSender<Message<T>>;
+pub type JobsReceiver<T> = mpsc::Receiver<Message<T>>;
 
+/// A pool of threads for executing work.
 pub struct ThreadPool<T> {
     pub jobs_tx: JobsSender<T>,
     workers:     Vec<Worker>,
 }
 
+/// Represents a message that Workers should respond to with a ResultPacket.
 pub enum Message<T> {
     Job(JobPacket<T>),
     Terminate,
 }
 
+/// Contains information required to complete a job.
 pub struct JobPacket<T> {
     pub job:       fn() -> T,
     pub return_tx: ResultsSender<T>,
 }
 
+/// Contains the result of an evaluated job.
 pub struct ResultPacket<T> {
     pub result: T,
 }
@@ -37,7 +42,8 @@ impl<T: Send + 'static> ThreadPool<T> {
     pub fn new(size: usize, monitor_tx: MonitorSender) -> Self {
         assert!(size > 0,);
 
-        let (jobs_tx, jobs_rx) = flume::bounded(100);
+        let (jobs_tx, jobs_rx) = mpsc::sync_channel(100);
+        let jobs_rx = Arc::new(Mutex::new(jobs_rx));
 
         let workers = (0..size)
             .map(|_| Worker::new(jobs_rx.clone(), monitor_tx.clone()))
@@ -45,20 +51,18 @@ impl<T: Send + 'static> ThreadPool<T> {
 
         ThreadPool { workers, jobs_tx }
     }
-
-    pub fn execute(&mut self, job: JobPacket<T>) {
-        let message = Message::Job(job);
-        self.jobs_tx.send(message).unwrap();
-    }
 }
 
+/// Represents a single worker thread.
 struct Worker {
     handle: Option<JoinHandle<()>>,
 }
 
 impl Worker {
+    /// Creates a new Worker that will listen for jobs on jobs_rx, respond on
+    /// the given return channel, and notify the framework via monitor_tx.
     pub fn new<T: Send + 'static>(
-        jobs_rx: JobsReceiver<T>, monitor_tx: MonitorSender,
+        jobs_rx: Arc<Mutex<JobsReceiver<T>>>, monitor_tx: MonitorSender,
     ) -> Self {
         let handle =
             Some(thread::spawn(move || Self::listen(jobs_rx, monitor_tx)));
@@ -66,22 +70,16 @@ impl Worker {
     }
 
     fn listen<T: Send + 'static>(
-        jobs_rx: JobsReceiver<T>, monitor_tx: MonitorSender,
+        jobs_rx: Arc<Mutex<JobsReceiver<T>>>, monitor_tx: MonitorSender,
     ) {
         loop {
-            let message = jobs_rx.recv().unwrap();
+            let message = jobs_rx.lock().unwrap().recv().unwrap();
 
             match message {
                 Message::Terminate => break,
                 Message::Job(JobPacket { job, return_tx }) => {
-                    // println!("worker {:?} got a job",
-                    // thread::current().id());
                     return_tx.send(ResultPacket { result: job() }).unwrap();
                     monitor_tx.send(Command::Refresh).unwrap();
-                    // println!(
-                    //     "worker {:?} finished a job",
-                    //     thread::current().id()
-                    // );
                 },
             }
         }
@@ -89,6 +87,8 @@ impl Worker {
 }
 
 impl<T> Drop for ThreadPool<T> {
+    // When the ThreadPool is dropped, tell each Worker to stop and collect
+    // their JoinHandles before continuing.
     fn drop(&mut self) {
         for _ in &self.workers {
             self.jobs_tx.send(Message::Terminate).unwrap();
@@ -110,9 +110,10 @@ mod tests {
     #[test]
     fn workers_can_terminate() {
         let (jobs_tx, jobs_rx): (JobsSender<String>, JobsReceiver<String>) =
-            flume::bounded(10);
+            mpsc::sync_channel(10);
+        let jobs_rx = Arc::new(Mutex::new(jobs_rx));
 
-        let (monitor_tx, _monitor_rx) = flume::unbounded();
+        let (monitor_tx, _monitor_rx) = mpsc::sync_channel(10);
         let worker = Worker::new(jobs_rx, monitor_tx);
 
         jobs_tx.send(Message::Terminate).unwrap();
@@ -121,11 +122,11 @@ mod tests {
 
     #[test]
     fn workers_return_correct_values() {
-        let (jobs_tx, jobs_rx): (JobsSender<String>, JobsReceiver<String>) =
-            flume::bounded(10);
-        let (results_tx, results_rx) = flume::bounded(10);
+        let (jobs_tx, jobs_rx) = mpsc::sync_channel(10);
+        let (results_tx, results_rx) = mpsc::sync_channel(10);
+        let jobs_rx = Arc::new(Mutex::new(jobs_rx));
 
-        let (monitor_tx, _monitor_rx) = flume::unbounded();
+        let (monitor_tx, _monitor_rx) = mpsc::sync_channel(10);
         let worker = Worker::new(jobs_rx, monitor_tx);
 
         jobs_tx
@@ -141,20 +142,4 @@ mod tests {
         jobs_tx.send(Message::Terminate).unwrap();
         worker.handle.unwrap().join().unwrap();
     }
-
-    #[test]
-    fn pool_returns_correct_values() {
-        let (results_tx, results_rx) = flume::bounded(10);
-
-        let (monitor_tx, _monitor_rx) = flume::unbounded();
-        let mut pool = ThreadPool::new(4, monitor_tx);
-
-        pool.execute(JobPacket {
-            job:       || String::from("test1"),
-            return_tx: results_tx,
-        });
-
-        let result = results_rx.recv().unwrap().result;
-        assert_eq!(result, "test1");
-    }
 }
